"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const xmlParser = require("fast-xml-parser");
const parseMpd = (mpdStr) => {
    const mpd = xmlParser.parse(mpdStr, {
        parseAttributeValue: true,
        attrNodeName: "attr",
        ignoreAttributes: false,
    });
    const result = {
        videoTracks: [],
        audioTracks: [],
        rawMpd: mpd,
    };
    const adaptationSets = mpd.MPD.Period.AdaptationSet;
    const videoAdaptation = adaptationSets.find((adaptation) => adaptation.attr["@_mimeType"].startsWith("video"));
    const audioAdaptation = adaptationSets.find((adaptation) => adaptation.attr["@_mimeType"].startsWith("audio"));
    const videoRepresentations = Array.isArray(videoAdaptation.Representation)
        ? videoAdaptation.Representation
        : [videoAdaptation.Representation];
    for (const videoRepresentation of videoRepresentations) {
        const track = {
            id: parseInt(videoRepresentation.attr["@_id"]),
            bitrate: parseInt(videoRepresentation.attr["@_bandwidth"]),
            urls: [],
        };
        const baseUrl = videoRepresentation.BaseURL;
        for (const segmentUrl of videoRepresentation.SegmentList.SegmentURL) {
            track.urls.push(baseUrl + segmentUrl.attr["@_media"]);
        }
        result.videoTracks.push(track);
    }
    const audioRepresentations = Array.isArray(audioAdaptation.Representation)
        ? audioAdaptation.Representation
        : [audioAdaptation.Representation];
    for (const audioRepresentation of audioRepresentations) {
        const track = {
            id: parseInt(audioRepresentation.attr["@_id"]),
            bitrate: parseInt(audioRepresentation.attr["@_bandwidth"]),
            urls: [],
        };
        const baseUrl = audioRepresentation.BaseURL;
        for (const segmentUrl of audioRepresentation.SegmentList.SegmentURL) {
            track.urls.push(baseUrl + segmentUrl.attr["@_media"]);
        }
        result.audioTracks.push(track);
    }
    result.videoTracks = result.videoTracks.sort((a, b) => b.bitrate - a.bitrate);
    result.audioTracks = result.audioTracks.sort((a, b) => b.bitrate - a.bitrate);
    return result;
};
exports.default = parseMpd;
//# sourceMappingURL=mpd_parser.js.map